package com.sonydna.common.extensions

import android.graphics.Bitmap
import java.io.File
import java.net.URLConnection

import com.sonydna.common.annotation.functional_programming.AsPure
import com.sonydna.common.annotation.functional_programming.Immutable
import com.sonydna.common.annotation.null_analysis.NonNullByDefault

import static extension com.sonydna.common.extensions.ScBitmap.ScConfig.*
import static extension com.sonydna.common.extensions.ScDimension.*

@Immutable
@NonNullByDefault
final class ScByteSize {
    public final long value

    private new(long e) { value = e }

    //	conversions ///////////////////////////////////////
    
    @AsPure def static final b  (long e) { new ScByteSize(e) }
    @AsPure def static final kb (long e) { new ScByteSize(e * 1024) }
    @AsPure def static final mb (long e) { new ScByteSize(e * 1024 * 1024) }
    
    def static final getByteSize (File          e) { new ScByteSize(e.length) }
    def static final getByteSize (Bitmap        e) { e.config.bitDepth * e.dimension.size }
    def static final getByteSize (URLConnection e) { new ScByteSize(e.contentLength) }

    //	xtend operator overload ///////////////////////////
    
    @AsPure def final operator_equals        (ScByteSize e) { value == e.value }
    @AsPure def final operator_notEquals     (ScByteSize e) { value != e.value }
    @AsPure def final operator_lessThan      (ScByteSize e) { value <  e.value }
    @AsPure def final operator_lessEqualThan (ScByteSize e) { value <= e.value }
    @AsPure def final operator_plus          (ScByteSize e) { new ScByteSize(value + e.value) }
    @AsPure def final operator_minus         (ScByteSize e) { new ScByteSize(value - e.value) }
    @AsPure def final operator_multiply      (long e)       { new ScByteSize(value * e) }
    @AsPure def final operator_divide        (long e)       { new ScByteSize(value / e) }
}
