package com.sonydna.common.extensions

import android.test.AndroidTestCase
import com.sonydna.common.annotation.null_analysis.NonNullByDefault

@NonNullByDefault
final class ScDimensionTest extends AndroidTestCase {

    def static void test_clone() {
        val a = new ScDimension(2, 4)
        assertTrue(new ScDimension(2, 4) == a)
        assertTrue(new ScDimension(2, 4) == a.clone)
    }

    def static void test_swap() {
        val a = new ScDimension(2, 4)
        val b = a.swap

        assertTrue(new ScDimension(2, 4) == a)
        assertTrue(new ScDimension(4, 2) == b)
    }

    def static void test_mul() {
        val a = new ScDimension(2, 4)
        val b = a * 2

        assertTrue(new ScDimension(2, 4) == a)
        assertTrue(new ScDimension(4, 8) == b)
    }

    def static void test_div() {
        val a = new ScDimension(2, 4)
        val b = a / 2

        assertTrue(new ScDimension(2, 4) == a)
        assertTrue(new ScDimension(1, 2) == b)
    }

    def static void test_fitInsideScale() {
        val a = new ScDimension(20, 25)
        val b = new ScDimension(10, 10)

        assertTrue(a.fitInsideScale(b) == 0.4f)
    }

    def static void test_fitOutsideScale() {
        val a = new ScDimension(20, 25)
        val b = new ScDimension(10, 10)

        assertTrue(a.fitOutsideScale(b) == 0.5f)
    }

    def static void test_fitInsideSize() {
        val a = new ScDimension(20, 25)
        val b = new ScDimension(10, 10)

        assertTrue(new ScDimension(8, 10) == a.fitInside(b))
    }

    def static void test_fitOutsideSize() {
        val a = new ScDimension(20, 25)
        val b = new ScDimension(10, 10)

        assertTrue(new ScDimension(10, 12) == a.fitOutside(b))
    }

    def static void test_insideSampleSize() {
        val vga = new ScDimension(640, 480)
        assertTrue(vga.insideSampleSize(new ScDimension(640, 480)) == 1) //  equal
        assertTrue(vga.insideSampleSize(new ScDimension(800, 800)) == 1) //  over
        assertTrue(vga.insideSampleSize(new ScDimension(320, 240)) == 2) //  half-size

        assertTrue(vga.insideSampleSize(new ScDimension(500, 500)) == 2) //  under
        assertTrue(vga.insideSampleSize(new ScDimension(320, 320)) == 2)
        assertTrue(vga.insideSampleSize(new ScDimension(300, 300)) == 4)
        assertTrue(vga.insideSampleSize(new ScDimension(100, 100)) == 8)
    }

    def static void test_outsideSampleSize() {
        val vga = new ScDimension(640, 480)
        assertTrue(vga.outsideSampleSize(new ScDimension(640, 480)) == 1) //  equal
        assertTrue(vga.outsideSampleSize(new ScDimension(800, 800)) == 1) //  over
        assertTrue(vga.outsideSampleSize(new ScDimension(320, 240)) == 2) //  half-size

        assertTrue(vga.outsideSampleSize(new ScDimension(500, 500)) == 1) //  under
        assertTrue(vga.outsideSampleSize(new ScDimension(320, 320)) == 1)
        assertTrue(vga.outsideSampleSize(new ScDimension(300, 300)) == 1)
        assertTrue(vga.outsideSampleSize(new ScDimension(100, 100)) == 4)
    }

    def static void test_getAspectRatio() {
        assertTrue(new ScDimension(1, 3).aspectRatio == 3)
        assertTrue(new ScDimension(2, 3).aspectRatio == 1.5)
        assertTrue(new ScDimension(3, 2).aspectRatio == 1.5)
        assertTrue(new ScDimension(3, 3).aspectRatio == 1)
    }
}
