package com.sonydna.common.extensions

import java.io.File

import android.test.AndroidTestCase
import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import java.util.ArrayList
import java.util.concurrent.Callable
import java.util.concurrent.Executors
import java.util.concurrent.Future
import org.apache.commons.io.FileUtils

import static extension com.sonydna.common.extensions.ScFile.*
import static extension com.sonydna.common.extensions.ScFile.ScExtension.*

class ScFileTest extends AndroidTestCase {
	
    def static void test_asFutureByCreateOrMovedTo_create() {

        val dir   = ScFile.externalStorageDirectory
        val file1 = new File(dir, "test1")

        file1.delete
        
        try {
            val future1 = ScFile.asFutureByCreateOrMovedTo(file1)
            assertNotNull(future1)
            
            file1.createNewFile
            future1.get

        } finally {
            file1.delete
        }
    }
    
    def static void test_asFutureByCreateOrMovedTo_movedTo() {

        val dir   = ScFile.getExternalStorageDirectory
        val file1 = new File(dir, "test1")
        val file2 = new File(dir, "test2")

        file1.delete
        file2.delete
        
        try {
            val future2 = ScFile.asFutureByCreateOrMovedTo(file2)
            assertNotNull(future2)
            
            file1.createNewFile
            file1.renameTo(file2)
            future2.get

        } finally {
            file1.delete
            file2.delete
        }
    }
    
    def static void test_asFutureByCreateOrMovedTo_instance2() {

        val dir   = ScFile.getExternalStorageDirectory
        val file1 = new File(dir, "test1")
        val file2 = new File(dir, "test2")

        file1.delete
        file2.delete
        
        try {
            val future1 = ScFile.asFutureByCreateOrMovedTo(file1)
            val future2 = ScFile.asFutureByCreateOrMovedTo(file2)
            assertNotNull(future1)
            assertNotNull(future2)
            
            file1.createNewFile
            file2.createNewFile
            future1.get
            future2.get

        } finally {
            file1.delete
            file2.delete
        }
    }
    
    def static void test_asFutureByCreateOrMovedTo_instance4() {

		val files = #[ "test1", "test2", "test3", "test4" ].map[ ScFile.getExternalStorageDirectory(it) ]
		files.forEach[ delete ]
		
		val deleteAll = files.map[ !it.exists ].fold(true, [e1, e2 | e1 && e2])
		assertTrue(deleteAll)
		
//		files.map では、このタイミングで asFutureByCreateOrMovedTo が呼ばれない…代わりに forEach を使う
//		val futures = files.map[ asFutureByCreateOrMovedTo ]
		val futures = new ArrayList<Future<File>>
		files.forEach[ futures.add(asFutureByCreateOrMovedTo) ]
		
		//	create new file on other threads
		val service = Executors.newFixedThreadPool(4)
		files.map [ new CreateFileTask(it) ].forEach[ service.submit(it) ]
		
		try {
			futures.forEach[ get ]
		} finally {
			files.forEach[ delete ]
		}
    }
    
    def static void test_makeNameUnique_01() {
        val dir = ScFile.getExternalStorageDirectory(String.valueOf(System.currentTimeMillis))

        val actual = new ArrayList<String>
        try {
            dir.mkdirs;
            (0..<3).map[ new File(dir, "name.ext") ].map[ makeNameUnique ].forEach[ createNewFile; actual.add(name) ]

        } finally {
            FileUtils.deleteDirectory(dir)
        }

        val expected = #["name.ext", "name (2).ext", "name (3).ext" ]
        assertEquals(expected, actual)
    }

    def static void test_makeNameUnique_02() {
        val dir = ScFile.getExternalStorageDirectory(String.valueOf(System.currentTimeMillis))

        val actual = new ArrayList<String>
        try {
            dir.mkdirs;
            (0..<3).map[ new File(dir, "name") ].map[ makeNameUnique ].forEach[ createNewFile; actual.add(name) ]

        } finally {
            FileUtils.deleteDirectory(dir)
        }

        val expected = #["name", "name (2)", "name (3)" ]
        assertEquals(expected, actual)
    }

    def static void test_isJpeg() {
        assertTrue (new File("name.jpg") .extension.jpeg)
        assertTrue (new File("name.jpeg").extension.jpeg)
        assertTrue (new File("name.JpEg").extension.jpeg)
        assertFalse(new File("name.png") .extension.jpeg)
    }
}

@NonNullByDefault
class CreateFileTask implements Callable<Boolean> {
	
	val File file
	
	new(File f) {
		file = f
	}
	
	override call() throws Exception {
		file.createNewFile
		true
	}
}
