package com.sonydna.common.extensions

import android.graphics.PointF
import android.graphics.RectF
import android.test.AndroidTestCase

import com.sonydna.common.annotation.null_analysis.NonNullByDefault

import static extension com.sonydna.common.extensions.ScRectF.*
import static extension com.sonydna.common.extensions.ScMatrix.*
import static extension com.sonydna.common.extensions.ScMatrix.*

@NonNullByDefault
final class ScMatrixTest extends AndroidTestCase {

    def static void test_asMatrixByCenterCrop_640x480() {
        val image  = new ScDimension(640, 480)
        val view   = new RectF(100, 100, 340, 340)
        val matrix = image.asMatrixByCenterCrop(view)

        val expected = new RectF(60, 100, 380, 340)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }

    def static void test_asMatrixByCenterCrop_480x640() {
        val image  = new ScDimension(480, 640)
        val view   = new RectF(100, 100, 340, 340)
        val matrix = image.asMatrixByCenterCrop(view)

        val expected = new RectF(100, 60, 340, 380)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }

    def static void test_asMatrixByInsideCrop_old_01() {
        val image  = new ScDimension(400, 200)
        val focus  = new PointF(0.5f, 0.5f)
        val view   = new RectF(0,0,100,100)
        val matrix = image.asMatrixByInsideCrop_old(focus, view)
        
        val expected = new RectF(-50f, 0f, 150f, 100f)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }

    def static void test_asMatrixByInsideCrop_old_02() {
        val image  = new ScDimension(100, 300)
        val focus  = new PointF(1f, 1f)
        val view   = new RectF(50,100,250,300)
        val matrix = image.asMatrixByInsideCrop_old(focus, view)
        
        val expected = new RectF(50, -300, 250, 300)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }

    def static void test_asMatrixByInsideCrop_01() {
        val image  = new ScDimension(400, 200)
        val focus  = new PointF(0.5f, 0.5f)
        val view   = new RectF(0,0,100,100)
        val matrix = image.asMatrixByInsideCrop(focus, view)
        
        val expected = new RectF(-50f, 0f, 150f, 100f)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }
    
    def static void test_asMatrixByInsideCrop_02() {
        val image  = new ScDimension(100, 300)
        val focus  = new PointF(1f, 1f)
        val view   = new RectF(50,100,250,300)
        val matrix = image.asMatrixByInsideCrop(focus, view)
        
        val expected = new RectF(50, -300, 250, 300)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }
    
    def static void test_asMatrixByCenterInside_640x480() {
        val image  = new ScDimension(640, 480)
        val view   = new RectF(100, 100, 340, 340)
        val matrix = image.asMatrixByCenterInside(view)

        val expected = new RectF(100, 130, 340, 310);
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }

    def static void test_asMatrixByCenterInside_480x640() {
        val image  = new ScDimension(480, 640)
        val view   = new RectF(100, 100, 340, 340)
        val matrix = image.asMatrixByCenterInside(view)

        val expected = new RectF(130, 100, 310, 340)
        assertTrue(image.asRectFByLeftTop(0, 0).map(matrix) == expected)
    }
}
