package com.sonydna.common.extensions

import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import android.test.AndroidTestCase
import android.graphics.RectF

import static extension com.sonydna.common.extensions.ScDimension.*

@NonNullByDefault
final class ScOrientationTest extends AndroidTestCase {
	
    def static void test01() {
        assertEquals(  0, ScOrientation._0.degrees)
        assertEquals( 90, ScOrientation._90.degrees)
        assertEquals(180, ScOrientation._180.degrees)
        assertEquals(270, ScOrientation._270.degrees)

        assertFalse(ScOrientation._0.is90or270)
        assertTrue (ScOrientation._90.is90or270)
        assertFalse(ScOrientation._180.is90or270)
        assertTrue (ScOrientation._270.is90or270)
    }

    def static void test_toMatrix() {
        val w = 160
        val h = 120
        val size = new ScDimension(w, h)
        {
            val matrix = ScOrientation._0.getMatrix(size)
            val rect = new RectF(0, 0, w, h)
            matrix.mapRect(rect)

            assertTrue(rect.left == 0 && rect.top == 0)
            assertTrue(rect.dimension == size)
        }

        {
            val matrix = ScOrientation._90.getMatrix(size)
            val rect = new RectF(0, 0, w, h)
            matrix.mapRect(rect)

            assertTrue(rect.left == 0 && rect.top == 0);
            assertTrue(rect.dimension == new ScDimension(h, w))
        }

        {
            val matrix = ScOrientation._180.getMatrix(size)
            val rect = new RectF(0, 0, size.w, size.h)
            matrix.mapRect(rect)

            assertTrue(rect.left == 0 && rect.top == 0)
            assertTrue(rect.dimension == size)
        }

        {
            val matrix = ScOrientation._270.getMatrix(size)
            val rect = new RectF(0, 0, w, h)
            matrix.mapRect(rect)

            assertTrue(rect.left == 0 && rect.top == 0)
            assertTrue(rect.dimension == new ScDimension(h, w))
        }
    }
}