package com.sonydna.common.extensions

import com.sonydna.common.annotation.functional_programming.AsPure
import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import android.graphics.Point
import android.graphics.PointF

import static extension com.sonydna.common.extensions.ScFloat.*
import android.view.MotionEvent

@NonNullByDefault
final class ScPointF {
	
    //	conversions ///////////////////////////////////////
    
    @AsPure def static final PointF asPointF(MotionEvent e) { new PointF(e.x, e.y) }
    @AsPure def static final PointF asPointF(Point       e) { new PointF(e.x, e.y) }
	
    //	xtend operator overload ///////////////////////////
    
    @AsPure def static final PointF operator_plus     (PointF e1, PointF e2) { new PointF(e1.x + e2.x, e1.y + e2.y) }
    @AsPure def static final PointF operator_minus    (PointF e1, PointF e2) { new PointF(e1.x - e2.x, e1.y - e2.y) }
    
    @AsPure def static final PointF operator_multiply (PointF p, int   e)   { new PointF(p.x * e, p.y * e) }
    @AsPure def static final PointF operator_divide   (PointF p, int   e)   { new PointF(p.x / e, p.y / e) }
    
    @AsPure def static final PointF operator_multiply (PointF p, float e)   { new PointF(p.x * e, p.y * e) }
    @AsPure def static final PointF operator_divide   (PointF p, float e)   { new PointF(p.x / e, p.y / e) }
    
    ///////////////////////////////////////////////////////
    
    def static final PointF clone(PointF e) { new PointF(e.x, e.y) }

    //	TODO: replace with ScMatrix.asMatrixByInsideCrop()
    def static final void insideCropCenter(ScDimension content, PointF center, int outerW, int outerH) {
        val adjustedCx = center.x.filter(ScRangeF.asRangeByFromTo(outerW / 2.0f, content.w - outerW / 2.0f))
        val adjustedCy = center.y.filter(ScRangeF.asRangeByFromTo(outerH / 2.0f, content.h - outerH / 2.0f))
        center.set(adjustedCx, adjustedCy)
    }
}
