package com.sonydna.common.extensions

import com.sonydna.common.annotation.functional_programming.AsPure
import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import android.graphics.Rect
import android.graphics.Point
import android.graphics.RectF
import static extension com.sonydna.common.extensions.ScPoint.*

@NonNullByDefault
final class ScRect {
	
    //	conversions ///////////////////////////////////////
    
    @AsPure def static final Rect asRectByLeftTop (ScDimension dimension, int left, int top) { new Rect(left, top, left + dimension.w, top + dimension.h) }
    @AsPure def static final Rect asRectByLeftTop (ScDimension dimension, Point leftTop)     { asRectByLeftTop(dimension, leftTop.x, leftTop.y) }
    @AsPure def static final Rect asRectByCenter  (ScDimension dimension, Point center)      { asRectByLeftTop(dimension, center - dimension.center) }
    @AsPure def static final Rect asRect          (RectF e)                                  { new Rect(e.left as int, e.top as int, e.right as int, e.bottom as int) }

    //	xtend operator overload ///////////////////////////
    
    @AsPure def static final operator_equals        (Rect e1, Rect e2) { (e1.left == e2.left) && (e1.top == e2.top) && (e1.right == e2.right) && (e1.bottom == e2.bottom) }
    @AsPure def static final operator_notEquals     (Rect e1, Rect e2) { (e1.left != e2.left) || (e1.top != e2.top) || (e1.right != e2.right) || (e1.bottom != e2.bottom) }
    @AsPure def static final operator_lessThan      (Rect e1, Rect e2) { (e2.left <  e1.left) && (e2.top <  e1.top) && (e1.right <  e2.right) && (e1.bottom <  e2.bottom) }
    @AsPure def static final operator_lessEqualThan (Rect e1, Rect e2) { (e2.left <= e1.left) && (e2.top <= e1.top) && (e1.right <= e2.right) && (e1.bottom <= e2.bottom) }
    
    @AsPure def static final operator_multiply (Rect r, int   e) { new Rect(r.left * e, r.top * e, r.right * e, r.bottom * e) }
    @AsPure def static final operator_divide   (Rect r, int   e) { new Rect(r.left / e, r.top / e, r.right / e, r.bottom / e) }
    
    @AsPure def static final operator_multiply (Rect r, float e) { new Rect((r.left * e) as int, (r.top * e) as int, (r.right * e) as int, (r.bottom * e) as int) }
    @AsPure def static final operator_divide   (Rect r, float e) { new Rect((r.left / e) as int, (r.top / e) as int, (r.right / e) as int, (r.bottom / e) as int) }
    
    ///////////////////////////////////////////////////////
    
    @AsPure def static final Rect clone(Rect e) { new Rect(e.left, e.top, e.right, e.bottom) }
    
    @AsPure def static final Point getCenter(Rect e) {
        new Point(e.left + e.width / 2, e.top + e.height / 2)
    }

    @AsPure def static final Rect offset(Rect e, Point point) {
        new Rect(e.left + point.x, e.top + point.y, e.right + point.x, e.bottom + point.y)
    }
    
    @AsPure def static final Rect inflate(Rect rect, int dx, int dy) {
    	new Rect(rect.left - dx, rect.top - dy, rect.right + dx, rect.bottom + dy)
    }
}