package com.sonydna.common.extensions

import com.sonydna.common.annotation.functional_programming.AsPure
import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import android.graphics.Rect
import android.graphics.Point
import android.graphics.RectF
import android.graphics.PointF
import android.graphics.Matrix
import static extension com.sonydna.common.extensions.ScPoint.*

@NonNullByDefault
final class ScRectF {
	
    //	conversions ///////////////////////////////////////
    
    @AsPure def static final RectF asRectFByLeftTop (ScDimension dimension, int left, int top) { new RectF(left, top, left + dimension.w, top + dimension.h) }
    @AsPure def static final RectF asRectFByLeftTop (ScDimension dimension, Point leftTop)     { asRectFByLeftTop(dimension, leftTop.x, leftTop.y) }
    @AsPure def static final RectF asRectFByCenter  (ScDimension dimension, Point center)      { asRectFByLeftTop(dimension, center - dimension.center) }
    @AsPure def static final RectF asRectF          (Rect e)                                   { new RectF(e.left, e.top, e.right, e.bottom) }

    //	xtend operator overload ///////////////////////////
    
    @AsPure def static final operator_equals        (RectF e1, RectF e2) { (e1.left == e2.left) && (e1.top == e2.top) && (e1.right == e2.right) && (e1.bottom == e2.bottom) }
    @AsPure def static final operator_notEquals     (RectF e1, RectF e2) { (e1.left != e2.left) || (e1.top != e2.top) || (e1.right != e2.right) || (e1.bottom != e2.bottom) }
    @AsPure def static final operator_lessThan      (RectF e1, RectF e2) { (e2.left <  e1.left) && (e2.top <  e1.top) && (e1.right <  e2.right) && (e1.bottom <  e2.bottom) }
    @AsPure def static final operator_lessEqualThan (RectF e1, RectF e2) { (e2.left <= e1.left) && (e2.top <= e1.top) && (e1.right <= e2.right) && (e1.bottom <= e2.bottom) }
    
    @AsPure def static final operator_multiply (RectF r, int   e) { new RectF(r.left * e, r.top * e, r.right * e, r.bottom * e) }
    @AsPure def static final operator_divide   (RectF r, int   e) { new RectF(r.left / e, r.top / e, r.right / e, r.bottom / e) }
    
    @AsPure def static final operator_multiply (RectF r, float e) { new RectF(r.left * e, r.top * e, r.right * e, r.bottom * e) }
    @AsPure def static final operator_divide   (RectF r, float e) { new RectF(r.left / e, r.top / e, r.right / e, r.bottom / e) }

    ///////////////////////////////////////////////////////
    
    @AsPure def static final RectF clone(RectF e) { new RectF(e.left, e.top, e.right, e.bottom) }
    
    @AsPure def static final PointF getCenter(RectF e) {
        new PointF(e.left + e.width / 2, e.top + e.height / 2)
    }
    
    @AsPure def static final RectF offset(RectF e, Point point) {
        new RectF(e.left + point.x, e.top + point.y, e.right + point.x, e.bottom + point.y)
    }
    
    @AsPure def static final RectF inflate(RectF rect, int dx, int dy) {
    	new RectF(rect.left - dx, rect.top - dy, rect.right + dx, rect.bottom + dy)
    }
    
    @AsPure def static final RectF map(RectF rect, Matrix m) {
    	val result = new RectF
    	m.mapRect(result, rect)
    	return result
    }
}