package com.sonydna.common.extensions

import android.graphics.Typeface

import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import com.sonydna.common.annotation.null_analysis.Nullable

import java.util.HashMap

//	Typeface は、一度生成すると 100% native-resource が leak する (native-resource には GC が効かない)
//	Bitmap.recycle() や ByteBuffer.clear() などの様な、native-resource を解放する手段がない為。
//	そのため Typeface は、 leak 前提で極力 cache する必要がある。

@NonNullByDefault
final class ScTypeface {
	
    val static cacheFromFile  = new HashMap<String, Typeface>
    val static cacheFromAsset = new HashMap<String, Typeface>

	@Nullable def static final Typeface asTypeface(String pathname) {
        val e = cacheFromFile.get(pathname) ?: Typeface.createFromFile(pathname)
        cacheFromFile.put(pathname, e)
        return e
	}
	
	@Nullable def static final Typeface asTypefaceByAsset(String pathname) {
        val e = cacheFromAsset.get(pathname) ?: Typeface.createFromAsset(ScApp.instance.assets, pathname)
        cacheFromAsset.put(pathname, e)
        return e
	}
}
