package com.sonydna.millionmoments.core.layout

import android.util.Log
import android.util.Xml
import com.sonydna.common.annotation.null_analysis.NonNullByDefault
import com.sonydna.common.extensions.ScApp
import com.sonydna.common.extensions.ScDimension
import java.io.IOException
import java.util.ArrayList
import org.xmlpull.v1.XmlPullParser
import org.xmlpull.v1.XmlPullParserException

@NonNullByDefault 
class PageLayoutList {
	private static val LOG_TAG = "PageLayoutList"
	
	val layoutList = new ArrayList<PageLayout>

	def getPageLayoutFromLayoutId(String layoutId) {
		layoutList.filter[it.mLayoutId == layoutId].head
	}
	
	def getPageLayoutListFromPhotoCount(int photoCount) {
		layoutList.filter[it.mUserPictureEntities.size == photoCount && it.mMonthTitleEntities.size <= 0 && !it.isCover].toList
	}
	
	def getCoverPageLayoutListFromPhotoCount(int photoCount) {
		layoutList.filter[it.mUserPictureEntities.size == photoCount && 0 < it.mMonthTitleEntities.size].toList
	}
	
	new (ScDimension viewDimension) {	
		val xmlPullParser
		= try {
			Xml.newPullParser.xmlSetInput
		} catch (IOException e){
			Log.d(LOG_TAG, "Error(IOException)")
			return
		} catch (XmlPullParserException e){
			Log.d(LOG_TAG, "Error(XmlPullException)")
			return
		}
		
		var eventType = xmlPullParser.eventType
		try {
			while (eventType != XmlPullParser.END_DOCUMENT) {
				if (eventType == XmlPullParser.START_TAG) {
					if (xmlPullParser.name == "Page") {
						layoutList.add(new PageLayout(viewDimension, xmlPullParser))
					}
				}
				eventType = xmlPullParser.next
			}
			
			//表紙
			layoutList.add(new PageLayout(PageKind.COVER, viewDimension))
			//背表紙
			layoutList.add(new PageLayout(PageKind.LASTPAGE, viewDimension))
			
		} catch (Exception e) {
			Log.d(LOG_TAG, "parse exception")
			e.printStackTrace
			return
		}
	}

	private def xmlSetInput(XmlPullParser xmlPullParser) {
		xmlPullParser.setInput(ScApp.getInstance.getAssets.open("PageLayout/design.xml"), "UTF-8")
		xmlPullParser
	}
	
	private def isCover(PageLayout pageLayout) {
		PageKind.COVER.toString == pageLayout.mLayoutId
	}
}
